@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2018 -- leonerd@leonerd.org.uk
 
 package IO::Async::Process;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.71';
+our $VERSION = '0.75';
 
 use Carp;
 
@@ -55,6 +55,29 @@ C<IO::Async::Process> - start and manage a child process
 
  $loop->run;
 
+Also accessible via the L<IO::Async::Loop/open_process> method:
+
+ $loop->open_process(
+    command => [ "/bin/ping", "-c4", "some.host" ],
+
+    stdout => {
+       on_read => sub {
+          my ( $stream, $buffref, $eof ) = @_;
+          while( $$buffref =~ s/^(.*)\n// ) {
+             print "PING wrote: $1\n";
+          }
+          return 0;
+       },
+    },
+
+    on_finish => sub {
+       my $process = shift;
+       my ( $exitcode ) = @_;
+       my $status = ( $exitcode >> 8 );
+       ...
+    },
+ );
+
 =head1 DESCRIPTION
 
 This subclass of L<IO::Async::Notifier> starts a child process, and invokes a
@@ -82,7 +105,7 @@ number and string values. After a successful C<exec()> call, this condition
 can no longer happen.
 
 Note that this has a different name and a different argument order from
-C<< Loop->open_child >>'s C<on_error>.
+C<< Loop->open_process >>'s C<on_error>.
 
 If this is not provided and the process exits with an exception, then
 C<on_finish> is invoked instead, being passed just the exit code.
@@ -93,7 +116,8 @@ field will be an empty string. It will however always be defined. This can be
 used to distinguish the two cases:
 
  on_exception => sub {
-    my ( $self, $exception, $errno, $exitcode ) = @_;
+    my $self = shift;
+    my ( $exception, $errno, $exitcode ) = @_;
 
     if( length $exception ) {
        print STDERR "The process died with the exception $exception " .
@@ -418,6 +442,7 @@ sub _prepare_fds
 
       if( $via == FD_VIA_PIPEREAD ) {
          my ( $myfd, $childfd ) = IO::Async::OS->pipepair or croak "Unable to pipe() - $!";
+         $myfd->blocking( 0 );
 
          $handle->configure( read_handle => $myfd );
 
@@ -426,6 +451,7 @@ sub _prepare_fds
       }
       elsif( $via == FD_VIA_PIPEWRITE ) {
          my ( $childfd, $myfd ) = IO::Async::OS->pipepair or croak "Unable to pipe() - $!";
+         $myfd->blocking( 0 );
          $write_only++;
 
          $handle->configure( write_handle => $myfd );
@@ -439,6 +465,7 @@ sub _prepare_fds
          # can ->close them properly
          my ( $myread, $childwrite ) = IO::Async::OS->pipepair or croak "Unable to pipe() - $!";
          my ( $childread, $mywrite ) = IO::Async::OS->pipepair or croak "Unable to pipe() - $!";
+         $_->blocking( 0 ) for $myread, $mywrite;
 
          $handle->configure( read_handle => $myread, write_handle => $mywrite );
 
@@ -448,6 +475,7 @@ sub _prepare_fds
       }
       elsif( $via == FD_VIA_SOCKETPAIR ) {
          my ( $myfd, $childfd ) = IO::Async::OS->socketpair( $opts->{family}, $opts->{socktype} ) or croak "Unable to socketpair() - $!";
+         $myfd->blocking( 0 );
 
          $opts->{prefork}->( $myfd, $childfd ) if $opts->{prefork};
 
@@ -517,10 +545,16 @@ sub _add_to_loop
 
    my $is_code = defined $self->{code};
 
-   $self->{finish_future} = Future->needs_all( @$finish_futures )
+   my $f = $self->finish_future;
+
+   $self->{_finish_future} = Future->needs_all( @$finish_futures )
       ->on_done( $self->_capture_weakself( sub {
          my $self = shift or return;
 
+         $self->debug_printf( "FINISH status=0x%04x%s", $exitcode,
+            join " ", '', ( $dollarbang ? '$!' : '' ), ( $dollarat ? '$@' : '' )
+         );
+
          $self->{exitcode} = $exitcode;
          $self->{dollarbang} = $dollarbang;
          $self->{dollarat}   = $dollarat;
@@ -536,6 +570,8 @@ sub _add_to_loop
                $self->invoke_event( on_finish => $exitcode );
          }
 
+         $f->done( $exitcode );
+
          $self->remove_from_parent;
       } ),
    );
@@ -544,7 +580,7 @@ sub _add_to_loop
 sub DESTROY
 {
    my $self = shift;
-   $self->{finish_future}->cancel if $self->{finish_future};
+   $self->{_finish_future}->cancel if $self->{_finish_future};
 }
 
 sub notifier_name
@@ -563,6 +599,23 @@ sub notifier_name
 
 =cut
 
+=head2 finish_future
+
+   $f = $process->finish_future
+
+I<Since version 0.75.>
+
+Returns a L<Future> that completes when the process finishes. It will yield
+the exit code from the process.
+
+=cut
+
+sub finish_future
+{
+   my $self = shift;
+   return $self->{finish_future} //= $self->loop->new_future;
+}
+
 =head2 pid
 
    $pid = $process->pid
@@ -782,7 +835,9 @@ data written by the process can be captured.
     command => [ "writing-program", "arguments" ],
     stdout => { into => \$stdout },
     on_finish => sub {
-       print "The process has finished, and wrote:\n";
+       my $process = shift;
+       my ( $exitcode ) = @_;
+       print "Process has exited with code $exitcode, and wrote:\n";
        print $stdout;
     }
  );
