@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2018 -- leonerd@leonerd.org.uk
 
 package IO::Async::Resolver;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Function );
 
-our $VERSION = '0.64';
+our $VERSION = '0.75';
 
 # Socket 2.006 fails to getaddrinfo() AI_NUMERICHOST properly on MSWin32
 use Socket 2.007 qw(
@@ -38,7 +38,7 @@ C<IO::Async::Resolver> - performing name resolutions asynchronously
 
 =head1 SYNOPSIS
 
-This object is used indirectly via an C<IO::Async::Loop>:
+This object is used indirectly via an L<IO::Async::Loop>:
 
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new;
@@ -63,12 +63,12 @@ This object is used indirectly via an C<IO::Async::Loop>:
 
 =head1 DESCRIPTION
 
-This module extends an C<IO::Async::Loop> to use the system's name resolver
+This module extends an L<IO::Async::Loop> to use the system's name resolver
 functions asynchronously. It provides a number of named resolvers, each one
 providing an asynchronous wrapper around a single resolver function.
 
 Because the system may not provide asynchronous versions of its resolver
-functions, this class is implemented using a C<IO::Async::Function> object
+functions, this class is implemented using a L<IO::Async::Function> object
 that wraps the normal (blocking) functions. In this case, name resolutions
 will be performed asynchronously from the rest of the program, but will likely
 be done by a single background worker process, so will be processed in the
@@ -76,7 +76,7 @@ order they were requested; a single slow lookup will hold up the queue of
 other requests behind it. To mitigate this, multiple worker processes can be
 used; see the C<workers> argument to the constructor.
 
-The C<idle_timeout> parameter for the underlying C<IO::Async::Function> object
+The C<idle_timeout> parameter for the underlying L<IO::Async::Function> object
 is set to a default of 30 seconds, and C<min_workers> is set to 0. This
 ensures that there are no spare processes sitting idle during the common case
 of no outstanding requests.
@@ -113,6 +113,34 @@ sub _init
    $started = 1;
 }
 
+sub debug_printf_call
+{
+   my $self = shift;
+   my ( $type, undef, @data ) = @_;
+
+   my $arg0;
+   if( $type eq "getaddrinfo" ) {
+      my %args = @data;
+      $arg0 = sprintf "%s:%s", @args{qw( host service )};
+   }
+   elsif( $type eq "getnameinfo" ) {
+      # cheat
+      $arg0 = sprintf "%s:%s", ( Socket::getnameinfo( $data[0], NI_NUMERICHOST|NI_NUMERICSERV ) )[1,2];
+   }
+   else {
+      $arg0 = $data[0];
+   }
+
+   $self->debug_printf( "CALL $type $arg0" );
+}
+
+sub debug_printf_result
+{
+   my $self = shift;
+   my ( @result ) = @_;
+   $self->debug_printf( "RESULT n=" . scalar @result );
+}
+
 =head1 METHODS
 
 The following methods documented with a trailing call to C<< ->get >> return
@@ -120,7 +148,9 @@ L<Future> instances.
 
 =cut
 
-=head2 @result = $loop->resolve( %params )->get
+=head2 resolve
+
+   @result = $loop->resolve( %params )->get
 
 Performs a single name resolution operation, as given by the keys in the hash.
 
@@ -145,7 +175,15 @@ with a timeout exception. If not supplied, a default of 10 seconds will apply.
 
 =back
 
-=head2 $resolver->resolve( %params )
+On failure, the fail category name is C<resolve>; the details give the
+individual resolver function name (e.g. C<getaddrinfo>), followed by other
+error details specific to the resolver in question.
+
+ ->fail( $message, resolve => $type => @details )
+
+=head2 resolve (void)
+
+   $resolver->resolve( %params )
 
 When not returning a future, additional parameters can be given containing the
 continuations to invoke on success or failure:
@@ -176,8 +214,8 @@ sub resolve
    my $type = $args{type};
    defined $type or croak "Expected 'type'";
 
-   if( $type eq "getaddrinfo" ) {
-      $type = "getaddrinfo_hash";
+   if( $type eq "getaddrinfo_hash" ) {
+      $type = "getaddrinfo";
    }
 
    exists $METHODS{$type} or croak "Expected 'type' to be an existing resolver method, got '$type'";
@@ -202,7 +240,10 @@ sub resolve
 
    my $future = $self->call(
       args => [ $type, $timeout, @{$args{data}} ],
-   );
+   )->else( sub {
+      my ( $message, @detail ) = @_;
+      Future->fail( $message, resolve => $type => @detail );
+   });
 
    $future->on_done( $on_resolved ) if $on_resolved;
    $future->on_fail( $on_error    ) if $on_error;
@@ -211,10 +252,12 @@ sub resolve
 
    # Caller is not going to keep hold of the Future, so we have to ensure it
    # stays alive somehow
-   $future->on_ready( sub { undef $future } ); # intentional cycle
+   $self->adopt_future( $future->else( sub { Future->done } ) );
 }
 
-=head2 @addrs = $resolver->getaddrinfo( %args )->get
+=head2 getaddrinfo
+
+   @addrs = $resolver->getaddrinfo( %args )->get
 
 A shortcut wrapper around the C<getaddrinfo> resolver, taking its arguments in
 a more convenient form.
@@ -257,6 +300,11 @@ each containing one result. Each result will contain fields called C<family>,
 C<socktype>, C<protocol> and C<addr>. If requested by C<AI_CANONNAME> then the
 C<canonname> field will also be present.
 
+On failure, the detail field will give the error number, which should match
+one of the C<Socket::EAI_*> constants.
+
+ ->fail( $message, resolve => getaddrinfo => $eai_errno )
+
 As a specific optimisation, this method will try to perform a lookup of
 numeric values synchronously, rather than asynchronously, if it looks likely
 to succeed.
@@ -266,7 +314,9 @@ like an IPv4 or IPv6 string, a synchronous lookup will first be performed
 using the C<AI_NUMERICHOST> flag. If this gives an C<EAI_NONAME> error, then
 the lookup is performed asynchronously instead.
 
-=head2 $resolver->getaddrinfo( %args )
+=head2 getaddrinfo (void)
+
+   $resolver->getaddrinfo( %args )
 
 When not returning a future, additional parameters can be given containing the
 continuations to invoke on success or failure:
@@ -301,7 +351,7 @@ sub getaddrinfo
       croak "Expected 'on_error' or to return a Future";
 
    my $host    = $args{host}    || "";
-   my $service = $args{service} || "";
+   my $service = $args{service}; defined $service or $service = "";
    my $flags   = $args{flags}   || 0;
 
    $flags |= AI_PASSIVE if $args{passive};
@@ -320,7 +370,7 @@ sub getaddrinfo
    # These address tests don't have to be perfect as if it fails we'll get
    # EAI_NONAME and just try it asynchronously anyway
    if( ( $host =~ m/^[\d.]+$/ or $host =~ m/^[[:xdigit:]:]$/ or $host eq "" ) and
-       $service =~ m/^\d+$/ ) {
+       $service =~ m/^\d*$/ ) {
 
        my ( $err, @results ) = Socket::getaddrinfo( $host, $service,
           { %args, flags => $flags | AI_NUMERICHOST }
@@ -342,7 +392,7 @@ sub getaddrinfo
    }
 
    my $future = $self->resolve(
-      type    => "getaddrinfo_hash",
+      type    => "getaddrinfo",
       data    => [
          host    => $host,
          service => $service,
@@ -350,10 +400,7 @@ sub getaddrinfo
          map { exists $args{$_} ? ( $_ => $args{$_} ) : () } qw( family socktype protocol ),
       ],
       timeout => $args{timeout},
-   )->else( sub {
-      my $message = shift;
-      Future->new->fail( $message, resolve => getaddrinfo => @_ );
-   });
+   );
 
    $future->on_done( $args{on_resolved} ) if $args{on_resolved};
    $future->on_fail( $args{on_error}    ) if $args{on_error};
@@ -362,10 +409,12 @@ sub getaddrinfo
 
    # Caller is not going to keep hold of the Future, so we have to ensure it
    # stays alive somehow
-   $future->on_ready( sub { undef $future } ); # intentional cycle
+   $self->adopt_future( $future->else( sub { Future->done } ) );
 }
 
-=head2 ( $host, $service ) = $resolver->getnameinfo( %args )->get
+=head2 getnameinfo
+
+   ( $host, $service ) = $resolver->getnameinfo( %args )->get
 
 A shortcut wrapper around the C<getnameinfo> resolver, taking its arguments in
 a more convenient form.
@@ -399,11 +448,18 @@ Time in seconds after which to abort the lookup with a C<Timed out> exception
 
 =back
 
+On failure, the detail field will give the error number, which should match
+one of the C<Socket::EAI_*> constants.
+
+ ->fail( $message, resolve => getnameinfo => $eai_errno )
+
 As a specific optimisation, this method will try to perform a lookup of
 numeric values synchronously, rather than asynchronously, if both the
 C<NI_NUMERICHOST> and C<NI_NUMERICSERV> flags are given.
 
-=head2 $resolver->getnameinfo( %args )
+=head2 getnameinfo (void)
+
+   $resolver->getnameinfo( %args )
 
 When not returning a future, additional parameters can be given containing the
 continuations to invoke on success or failure:
@@ -467,10 +523,7 @@ sub getnameinfo
       timeout => $args{timeout},
    )->transform(
       done => sub { @{ $_[0] } }, # unpack the ARRAY ref
-   )->else( sub {
-      my $message = shift;
-      Future->new->fail( $message, resolve => getnameinfo => @_ );
-   });
+   );
 
    $future->on_done( $args{on_resolved} ) if $args{on_resolved};
    $future->on_fail( $args{on_error}    ) if $args{on_error};
@@ -479,7 +532,7 @@ sub getnameinfo
 
    # Caller is not going to keep hold of the Future, so we have to ensure it
    # stays alive somehow
-   $future->on_ready( sub { undef $future } ); # intentional cycle
+   $self->adopt_future( $future->else( sub { Future->done } ) );
 }
 
 =head1 FUNCTIONS
@@ -561,12 +614,12 @@ register_resolver getprotobynumber => sub { my @r = getprotobynumber( $_[0] ) or
 
 The following three resolver names are implemented using the L<Socket> module.
 
- getaddrinfo_hash
+ getaddrinfo
  getaddrinfo_array
  getnameinfo
 
-The C<getaddrinfo_hash> resolver takes arguments in a hash of name/value pairs
-and returns a list of hash structures, as the C<Socket::getaddrinfo> function
+The C<getaddrinfo> resolver takes arguments in a hash of name/value pairs and
+returns a list of hash structures, as the C<Socket::getaddrinfo> function
 does. For neatness it takes all its arguments as named values; taking the host
 and service names from arguments called C<host> and C<service> respectively;
 all the remaining arguments are passed into the hints hash. This name is also
@@ -591,7 +644,7 @@ programs to be fully IPv6-capable.
 
 =cut
 
-register_resolver getaddrinfo_hash => sub {
+register_resolver getaddrinfo => sub {
    my %args = @_;
 
    my $host    = delete $args{host};
@@ -605,7 +658,7 @@ register_resolver getaddrinfo_hash => sub {
 
    my ( $err, @addrs ) = Socket::getaddrinfo( $host, $service, \%args );
 
-   die "$err\n" if $err;
+   die [ "$err", $err+0 ] if $err;
 
    return @addrs;
 };
@@ -624,7 +677,7 @@ register_resolver getaddrinfo_array => sub {
 
    my ( $err, @addrs ) = Socket::getaddrinfo( $host, $service, \%hints );
 
-   die "$err\n" if $err;
+   die [ "$err", $err+0 ] if $err;
 
    # Convert the @addrs list into a list of ARRAY refs of 5 values each
    return map {
@@ -637,7 +690,7 @@ register_resolver getnameinfo => sub {
 
    my ( $err, $host, $service ) = Socket::getnameinfo( $addr, $flags || 0 );
 
-   die "$err\n" if $err;
+   die [ "$err", $err+0 ] if $err;
 
    return [ $host, $service ];
 };
@@ -650,7 +703,7 @@ would likely make calls to OS functions to provide an answer. In traditional
 Unix style, a pair of functions are provided that each look up the entity by
 either type of key, where both functions return the same type of list. This is
 purely a convention, and is in no way required or enforced by the
-C<IO::Async::Resolver> itself.
+L<IO::Async::Resolver> itself.
 
  @numbers = qw( zero  one   two   three four
                 five  six   seven eight nine  );
@@ -669,16 +722,6 @@ C<IO::Async::Resolver> itself.
     die "Bad name $name";
  };
 
-=head1 TODO
-
-=over 4
-
-=item *
-
-Look into (system-specific) ways of accessing asynchronous resolvers directly
-
-=back
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
